@@ -1,9 +1,9 @@
 package Test::WWW::Stub;
-use 5.008001;
+use 5.010;
 use strict;
 use warnings;
 
-our $VERSION = "0.02";
+our $VERSION = "0.03";
 
 use Carp ();
 use Guard;  # guard
@@ -13,10 +13,49 @@ use Test::More ();
 use List::MoreUtils ();
 use URI;
 
-# PSGI app の場合は第2引数に Plack::Request でラップされたのもついてきます
 our $Handlers = { };
 our @Requests;
 
+my $register_g;
+my $app;
+sub _app { $app; }
+
+$app = sub {
+    my ($env) = @_;
+    my $req = Plack::Request->new($env);
+
+    push @Requests, $req;
+
+    my $uri = _normalize_uri($req->uri);
+
+    for my $key (keys %$Handlers) {
+        my $handler = $Handlers->{$key};
+        my @match;
+        if ($handler->{type} eq 'Regexp' ? (@match = ($uri =~ qr<$key>)) : $uri eq $key) {
+            if (my $app = $handler->{app}) {
+                $env->{'test.www.stub.handler'} = [ $key, $app ];
+                my $res = $app->($env, $req, @match);
+                return $res if $res;
+            } elsif (my $res = $handler->{res}) {
+                return $res;
+            } else {
+                Test::More::BAIL_OUT 'Handler MUST be a PSGI app or an ARRAY';
+            }
+        }
+    }
+
+    my ($file, $line) = _trace_file_and_line();
+
+    my $method = $req->method;
+    Test::More::diag "Unexpected external access: $method $uri at $file line $line";
+
+    return [ 499, [], [] ];
+};
+
+sub import {
+    $register_g = LWP::Protocol::PSGI->register($app);
+}
+
 sub register {
     my ($class, $uri_or_re, $app_or_res) = @_;
     $app_or_res //= [200, [], []];
@@ -42,6 +81,27 @@ sub last_request {
     return $Requests[-1];
 }
 
+sub last_request_for {
+    my ($class, $method, $url) = @_;
+    my $reqs = { map { _request_signature($_) => $_ } @Requests };
+    my $signature = "$method $url";
+    return $reqs->{$signature};
+}
+
+sub _request_signature {
+    my ($req) = @_;
+    my $normalized = _normalize_uri($req->uri);
+    return join ' ', $req->method, $normalized;
+}
+
+# Don't use query part of URI for handler matching.
+sub _normalize_uri {
+    my ($uri) = @_;
+    my $cloned = $uri->clone;
+    $cloned->query(undef);
+    return $cloned;
+}
+
 sub requests { @Requests }
 
 sub requested_ok {
@@ -49,8 +109,7 @@ sub requested_ok {
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     Test::More::ok(
         List::MoreUtils::any(sub {
-            my $req_url = $_->uri->clone;
-               $req_url->path_query($req_url->path);
+            my $req_url = _normalize_uri($_->uri);
             $_->method eq $method && $req_url eq $url
         }, @Requests),
         "stubbed $method $url",
@@ -61,44 +120,15 @@ sub clear_requests {
     @Requests = ();
 }
 
-my $app = sub {
-    my ($env) = @_;
-    my $req = Plack::Request->new($env);
-
-    push @Requests, $req;
-
-    # クエリは app で処理してほしい
-    my $uri = $req->uri->clone;
-       $uri->path_query($uri->path);
-
-    for my $key (keys %$Handlers) {
-        my $handler = $Handlers->{$key};
-        my @match;
-        if ($handler->{type} eq 'Regexp' ? (@match = ($uri =~ qr<$key>)) : $uri eq $key) {
-            if (my $app = $handler->{app}) {
-                $env->{'test.www.stub.handler'} = [ $key, $app ];
-                my $res = $app->($env, $req, @match);
-                return $res if $res;
-            } elsif (my $res = $handler->{res}) {
-                return $res;
-            } else {
-                Test::More::BAIL_OUT 'Handler MUST be a PSGI app or an ARRAY';
-            }
-        }
-    }
-
+sub _trace_file_and_line {
     my $level = $Test::Builder::Level;
     my (undef, $file, $line) = caller($level);
+    # assume "Actual" caller is test file named FOOBAR.t
     while ($file && $file !~ m<\.t$>) {
         (undef, $file, $line) = caller(++$level);
     }
-    my $method = $req->method;
-    Test::More::diag "Unexpected external access: $method $uri at $file line $line";
-
-    return [ 499, [], [] ];
-};
-
-my $register_g = LWP::Protocol::PSGI->register($app);
+    ($file, $line);
+}
 
 sub unstub {
     Carp::croak 'guard is required' unless defined wantarray;
@@ -119,7 +149,8 @@ Test::WWW::Stub - Block and stub specified URL for LWP
 
 =head1 SYNOPSIS
 
-    # External http(s) access via LWP is blocked by just using this module.
+    # External http(s) access via LWP is blocked by just declaring 'use Test::WWW::Stub';
+    # Note that 'require Test::WWW::Stub' or 'use Test::WWW::Stub ()' doesn't block external access.
     use Test::WWW::Stub;
 
     my $ua = LWP::UserAgent->new;
@@ -150,7 +181,7 @@ Test::WWW::Stub - Block and stub specified URL for LWP
 
     {
         # you can unstub and allow external access temporary
-        my $unregister_guard = Test::WWW::Stub->unregister;
+        my $unstub_guard = Test::WWW::Stub->unstub;
 
         # External access occurs!!
         ok $ua->get('http://example.com');
@@ -181,7 +212,11 @@ If called in void context, it simply registers the stub.
 Otherwise,it returns a new guard which drops the stub on destroyed.
 
 C<$uri_or_re> is either an URI string or a compiled regular expression for URI.
-C<$app_or_res> is a PSGI response array ref, or code ref which returns a PSGI response array ref.
+C<$app_or_res> is a PSGI response array ref, or a code ref which returns a PSGI response array ref.
+If C<$app_or_res> is a code ref, requests are passed to the code ref following syntax:
+
+    my $req = Plack::Request->new($env);
+    $app_or_res->($env, $req);
 
 Once registered, C<$app_or_res> will be return from LWP::UserAgent on requesting certain URI matches C<$uri_or_re>.
 
@@ -207,6 +242,12 @@ Returns a Plack::Request object last handled by Test::WWW::Stub.
 
 This method is same as C<[Test::WWW::Stub-E<gt>requests]-E<gt>[-1]>.
 
+=item  C<last_request_for>
+
+    my $last_req = Test::WWW::Stub->last_request_for($method, $uri);
+
+Returns a C<Plack::Request> object last handled by Test::WWW::Stub and matched given HTTP method and URI.
+
 =item C<clear_requests>
 
     Test::WWW::Stub->clear_requests;
